// You will need:
// Button library: https://github.com/JChristensen/Button
// OLED library: http://www.geekonfire.com/wiki/images/7/72/GOFi2cOLED.zip

#include <Wire.h>
#include <GOFi2cOLED.h>
#include <Button.h>

GOFi2cOLED GOFoled;

#define InVolt A7 // use different pin if you do not have A7 available on your board
// #define OutSel 8

unsigned int atten=0; //attenuator value
float prange=50; // power range - O2
float voltage = 0.0;
float freq = 2500;
double dbm = 0.0;
double mw = 0.0;
double ref = 0.0;
String att_out;
String frange_out;
String raw_out;
String mw_out;
String dbm_out;

Button BUp = Button(2,true,true,6);
Button BDown = Button(3,true,true,6);
Button BLeft = Button(4,true,true,6);
Button BRight = Button(5,true,true,6);
Button BHold = Button(6,true,true,6);

// set to different frequencies if you have calibration data available:
int frange_val[] = {100,600,1150,1870,2300,2500}; // calibrated at freq

// use different set of values if you got different attenuators
int atten_val[] = {0,6,10,18,26,28,32,36,44}; // attached attenuator values
// be sure to change values here if you have different calibration data
float ref_values[] = {1.67,1.69,1.667,1.86,1.812,1.705}; // output values at 0dBm/O3, at frequency...

int atten_mem = 0;
int frange_mem = 5;
int inval = 0;
char voltage_c[15]; // converted voltage from ADC
char dbm_c[15];
char mw_c[15];
char freq_c[15];

void setup()
{
  GOFoled.init(0x3C);
  GOFoled.setTextWrap(0);
  GOFoled.setHorizontalMode();
  GOFoled.clearDisplay();
  GOFoled.setTextColor(WHITE);
  
}

void loop()
{
  // =========================== BUTTONS =====================================
  BUp.read();
  BDown.read();
  BLeft.read();
  BRight.read();
  BHold.read();
  
   if(BDown.wasReleased()){
   frange_mem=frange_mem-1;
   if (frange_mem < 0) {
     frange_mem = 0;
   }
   freq = frange_val[frange_mem];
 } 
  if(BUp.wasReleased()){
   frange_mem=frange_mem+1;
   if (frange_mem > 5) {
     frange_mem = 5;
   }
   freq = frange_val[frange_mem];
 }
 if(BLeft.wasReleased()){
   atten_mem=atten_mem-1;
   if (atten_mem < 0) {
     atten_mem = 0;
   }
   atten = atten_val[atten_mem];
 } 
 if(BRight.wasReleased()){
   atten_mem=atten_mem+1;
   if (atten_mem > 8) {
     atten_mem = 8;
   }
   atten = atten_val[atten_mem];
 } 

  // ========================= VOLTAGE AND CALC ==============================

inval = analogRead(InVolt);
voltage = (0.00498 * inval); // 10-bit ADC sucks
dtostrf(voltage,7,5,voltage_c);

dtostrf(freq,4,0,freq_c);

// base ref: 1.705V @ 2.5GHz, 0dBm - O3
// x2.77@O2
// x5.55@O1
// zero ~0.5V@O3
// P(mW) = 1mW · 10(P(dBm)/ 10)

ref = (ref_values[frange_mem])*2.77;
dbm = ((ref - voltage)/(prange/1000) - atten) * -1;
if (dbm < -99)
  {
     dbm=-100;
  }
mw = pow(10,(dbm/10));
dtostrf(dbm,4,1,dbm_c);
dtostrf(mw,6,1,mw_c);

  
  // ========================= DISPLAY =======================================



  String at_b = "dB Att";
  String prange_out="50mV/dBm, ";  
  att_out=prange_out + atten + at_b;
  
  String fr_a = "Freq: ";
  String fr_b = "MHz";
  frange_out=fr_a + freq_c + fr_b;  
  
  String raw_a = "RAW V: ";
  raw_out = raw_a + voltage_c;
  
  String dbm_a = "dBm\n";
  dbm_out = dbm_c + dbm_a;
  
  String mw_a = "mW\n";
  mw_out = mw_c + mw_a;

  GOFoled.clearDisplay();
  GOFoled.setTextSize(1);
  GOFoled.setCursor(0,0);
  GOFoled.println(att_out);
  GOFoled.println(frange_out);
  GOFoled.setTextSize(2);
  GOFoled.print(mw_out);
  GOFoled.print(dbm_out);
  GOFoled.setTextSize(1);
  GOFoled.print(raw_out);
  GOFoled.display();

  delay (50);

}

